@@ -1,5 +1,4 @@
 /* Command line processing */
-/* $Id: cmdline.c,v 1.107.2.6 2005/05/02 20:34:59 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,9 +30,9 @@
 #include "config/options.h"
 #include "config/opttypes.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/dns.h"
+#include "network/dns.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "util/error.h"
 #include "util/file.h"
 #include "util/lists.h"
@@ -91,7 +90,7 @@ parse_options_(int argc, unsigned char *argv[], struct option *opt,
 				if (err[0]) {
 					usrerror(gettext("Cannot parse option %s: %s"), argv[-1], err);
 
-					return RET_ERROR;
+					return RET_SYNTAX;
 				}
 
 				/* XXX: Empty strings means all is well and have
@@ -113,7 +112,7 @@ parse_options_(int argc, unsigned char *argv[], struct option *opt,
 
 unknown_option:
 	usrerror(gettext("Unknown option %s"), argv[-1]);
-	return RET_ERROR;
+	return RET_SYNTAX;
 }
 
 enum retval
@@ -134,7 +133,7 @@ eval_cmd(struct option *o, unsigned char ***argv, int *argc)
 
 	(*argv)++; (*argc)--;	/* Consume next argument */
 
-	parse_config_file(config_options, "-eval", *(*argv - 1), NULL);
+	parse_config_file(config_options, "-eval", *(*argv - 1), NULL, 0);
 
 	fflush(stdout);
 
@@ -158,7 +157,7 @@ lookup_cmd(struct option *o, unsigned char ***argv, int *argc)
 	if (*argc > 1) return gettext("Too many parameters");
 
 	(*argv)++; (*argc)--;
-	if (do_real_lookup(*(*argv - 1), &addrs, &addrno, 0)) {
+	if (do_real_lookup(*(*argv - 1), &addrs, &addrno, 0) == DNS_ERROR) {
 #ifdef HAVE_HERROR
 		herror(gettext("error"));
 #else
@@ -208,59 +207,131 @@ remote_cmd(struct option *o, unsigned char ***argv, int *argc)
 			REMOTE_METHOD_PING,
 			REMOTE_METHOD_XFEDOCOMMAND,
 			REMOTE_METHOD_ADDBOOKMARK,
+			REMOTE_METHOD_INFOBOX,
 			REMOTE_METHOD_NOT_SUPPORTED,
 		} type;
 	} remote_methods[] = {
 		{ "openURL",	  REMOTE_METHOD_OPENURL },
 		{ "ping",	  REMOTE_METHOD_PING },
 		{ "addBookmark",  REMOTE_METHOD_ADDBOOKMARK },
+		{ "infoBox",	  REMOTE_METHOD_INFOBOX },
 		{ "xfeDoCommand", REMOTE_METHOD_XFEDOCOMMAND },
 		{ NULL,		  REMOTE_METHOD_NOT_SUPPORTED },
 	};
-	unsigned char *command, *arg, *argend = NULL;
+	unsigned char *command, *arg, *argend, *argstring;
 	int method, len = 0;
+	unsigned char *remote_argv[10];
+	int remote_argc;
 
 	if (*argc < 1) return gettext("Parameter expected");
 
 	command = *(*argv);
 
-	while (isalpha(command[len]))
+	while (isasciialpha(command[len]))
 		len++;
 
-	arg = strchr(&command[len], '(');
-	if (arg) {
-		arg++;
-		skip_space(arg);
+	/* Find the begining and end of the argument list. */
 
-		argend = strchr(arg, ')');
-	}
+	arg = command + len;
+	skip_space(arg);
 
-	if (!argend) {
+	argend = arg + strlen(arg);
+	skipback_whitespace(arg, argend);
+	if (argend > arg)
+		argend--;
+
+	/* Decide whether to use the "extended" --remote format where
+	 * all URLs following should be opened in tabs. */
+	if (len == 0 || *arg != '(' || *argend != ')') {
 		/* Just open any passed URLs in new tabs */
 		remote_session_flags |= SES_REMOTE_NEW_TAB;
 		return NULL;
 	}
 
-	skipback_whitespace(arg, argend);
+	arg++;
+
+	arg = argstring = memacpy(arg, argend - arg);
+	if (!argstring)
+		return gettext("Out of memory");
+
+	remote_argc = 0;
+	do {
+		unsigned char *start, *end;
+
+		if (remote_argc > sizeof_array(remote_argv)) {
+			mem_free(argstring);
+			return gettext("Too many arguments");
+		}
+
+		/* Skip parenthesis, comma, and surrounding whitespace. */
+		skip_space(arg);
+		start = arg;
+
+		if (*start == '"') {
+			end = ++start;
+			while ((end = strchr(end, '"'))) {
+				/* Treat "" inside quoted arg as ". */
+				if (end[1] != '"')
+					break;
+
+				end += 2;
+			}
+
+			if (!end)
+				return gettext("Mismatched ending argument quoting");
+
+			arg = end + 1;
+			skip_space(arg);
+			if (*arg && *arg != ',')
+				return gettext("Garbage after quoted argument");
+
+			remote_argv[remote_argc++] = start;
+			*end = 0;
+
+			/* Unescape "" to ". */
+			for (end = start; *end; start++, end++) {
+				*start = *end;
+				if (*end == '"')
+					end++;
+			}
+			*start = 0;
+
+		} else {
+			end = strchr(start, ',');
+			if (!end) {
+				end = start + strlen(start);
+				arg = end;
+			} else {
+				arg = end + 1;
+			}
+			skipback_whitespace(start, end);
+
+			if (start != end)
+				remote_argv[remote_argc++] = start;
+			*end = 0;
+		}
+
+		if (*arg == ',')
+			arg++;
+	} while (*arg);
 
 	for (method = 0; remote_methods[method].name; method++) {
 		unsigned char *name = remote_methods[method].name;
 
-		if (!strlcasecmp(command, len, name, -1))
+		if (!c_strlcasecmp(command, len, name, -1))
 			break;
 	}
 
 	switch (remote_methods[method].type) {
 	case REMOTE_METHOD_OPENURL:
-		if (arg == argend) {
+		if (remote_argc < 1) {
 			/* Prompt for a URL with a dialog box */
 			remote_session_flags |= SES_REMOTE_PROMPT_URL;
 			break;
 		}
 
-		len = strcspn(arg, ",)");
-		if (arg[len] == ',') {
-			unsigned char *where = arg + len + 1;
+		if (remote_argc == 2) {
+			unsigned char *where = remote_argv[1];
 
 			if (strstr(where, "new-window")) {
 				remote_session_flags |= SES_REMOTE_NEW_WINDOW;
@@ -278,15 +349,14 @@ remote_cmd(struct option *o, unsigned char ***argv, int *argc)
 			remote_session_flags |= SES_REMOTE_CURRENT_TAB;
 		}
 
-		while (len > 0 && isspace(arg[len - 1])) len--;
-		if (len) remote_url = memacpy(arg, len);
-
+		remote_url = stracpy(remote_argv[0]);
 		break;
 
 	case REMOTE_METHOD_XFEDOCOMMAND:
-		len = argend - arg;
+		if (remote_argc < 1)
+			break;
 
-		if (!strlcasecmp(arg, len, "openBrowser", 11)) {
+		if (!c_strcasecmp(remote_argv[0], "openBrowser")) {
 			remote_session_flags = SES_REMOTE_NEW_WINDOW;
 		}
 		break;
@@ -296,15 +366,27 @@ remote_cmd(struct option *o, unsigned char ***argv, int *argc)
 		break;
 
 	case REMOTE_METHOD_ADDBOOKMARK:
-		if (arg == argend) break;
-		remote_url = memacpy(arg, argend - arg);
+		if (remote_argc < 1)
+			break;
+		remote_url = stracpy(remote_argv[0]);
 		remote_session_flags = SES_REMOTE_ADD_BOOKMARK;
 		break;
 
+	case REMOTE_METHOD_INFOBOX:
+		if (remote_argc < 1)
+			break;
+		remote_url = stracpy(remote_argv[0]);
+		if (remote_url)
+			insert_in_string(&remote_url, 0, "about:", 6);
+		remote_session_flags = SES_REMOTE_INFO_BOX;
+		break;
+
 	case REMOTE_METHOD_NOT_SUPPORTED:
 		break;
 	}
 
+	mem_free(argstring);
+
 	/* If no flags was applied it can only mean we are dealing with
 	 * unknown method. */
 	if (!remote_session_flags)
@@ -359,7 +441,7 @@ print_full_help(struct option *tree, unsigned char *path)
 		    || (option->flags & OPT_HIDDEN))
 			continue;
 
-		if (!capt && !strncasecmp(option->name, "_template_", 10))
+		if (!capt && !c_strncasecmp(option->name, "_template_", 10))
 			capt = (unsigned char *) N_("Template option folder");
 
 		if (!capt) {
@@ -406,7 +488,7 @@ print_full_help(struct option *tree, unsigned char *path)
 
 			case OPT_COLOR:
 			{
-				color_t color = option->value.color;
+				color_T color = option->value.color;
 				unsigned char hexcolor[8];
 
 				printf(gettext("(default: %s)"),
@@ -418,7 +500,7 @@ print_full_help(struct option *tree, unsigned char *path)
 				break;
 
 			case OPT_LANGUAGE:
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 				printf(gettext("(default: \"%s\")"),
 				       language_to_name(option->value.number));
 #endif
@@ -473,7 +555,7 @@ print_full_help(struct option *tree, unsigned char *path)
 }
 
 static void
-print_short_help()
+print_short_help(void)
 {
 #define ALIGN_WIDTH 20
 	struct option *option;
@@ -547,7 +629,7 @@ printhelp_cmd(struct option *option, unsigned char ***argv, int *argc)
 		print_full_help(config_options, "");
 	} else {
 		printf("%s\n\n%s:\n",
-		       gettext("Usage: elinks [OPTION]... [URL]"),
+		       gettext("Usage: elinks [OPTION]... [URL]..."),
 		       gettext("Options"));
 		if (!strcmp(option->name, "long-help")) {
 			print_full_help(cmdline_options, "-");
@@ -741,13 +823,13 @@ struct option_info cmdline_options_info[] = {
 		"configuration directory (~/.elinks). It forces default configuration\n"
 		"values to be used and disables saving of runtime state files.")),
 
-	INIT_OPT_BOOL("", N_("Disable link numbering in dump output"),
-		"no-numbering", 0, 0,
+	INIT_OPT_CMDALIAS("", N_("Disable link numbering in dump output"),
+		"no-numbering", OPT_ALIAS_NEGATE, "document.dump.numbering",
 		N_("Prevents printing of link number in dump output.\n"
 		"Note that this really affects only -dump, nothing else.")),
 
-	INIT_OPT_BOOL("", N_("Disable printing of link references in dump output"),
-		"no-references", 0, 0,
+	INIT_OPT_CMDALIAS("", N_("Disable printing of link references in dump output"),
+		"no-references", OPT_ALIAS_NEGATE, "document.dump.references",
 		N_("Prevents printing of references (URIs) of document links\n"
 		"in dump output.\n"
 		"Note that this really affects only -dump, nothing else.")),
@@ -767,6 +849,7 @@ struct option_info cmdline_options_info[] = {
 		"\topenURL(URL, new-tab)     : open URL in new tab\n"
 		"\topenURL(URL, new-window)  : open URL in new window\n"
 		"\taddBookmark(URL)          : bookmark URL\n"
+		"\tinfoBox(text)             : show text in a message box\n"
 		"\txfeDoCommand(openBrowser) : open new window")),
 
 	INIT_OPT_INT("", N_("Connect to session ring with given ID"),
