@@ -1,5 +1,3 @@
-/* $Id: conf.h,v 1.14.6.1 2005/05/01 22:10:08 jonas Exp $ */
-
 #ifndef EL__CONFIG_CONF_H
 #define EL__CONFIG_CONF_H
 
@@ -16,11 +14,12 @@ enum parse_error {
 };
 
 void load_config(void);
-enum parse_error parse_config_command(struct option *options,
-				      unsigned char **file, int *line,
-				      struct string *mirror);
+#ifdef CONFIG_EXMODE
+enum parse_error parse_config_exmode_command(unsigned char *cmd);
+#endif
 void parse_config_file(struct option *options, unsigned char *name,
-		       unsigned char *file, struct string *mirror);
+		       unsigned char *file, struct string *mirror,
+		       int is_system_conf);
 int write_config(struct terminal *);
 
 unsigned char *
