@@ -157,7 +157,14 @@ debug_check_option_syntax(struct option *option)
 /* Ugly kludge */
 static int no_autocreate = 0;
 
-/* Get record of option of given name, or NULL if there's no such option. */
+/** Get record of option of given name, or NULL if there's no such option.
+ *
+ * If the specified option is an ::OPT_ALIAS, this function returns the
+ * alias, rather than the option to which the alias refers.  It must
+ * work this way because the alias may have the ::OPT_ALIAS_NEGATE flag.
+ * Instead, if the caller tries to read or set the value of the alias,
+ * the functions associated with ::OPT_ALIAS will forward the operation
+ * to the underlying option.  However, see indirect_option().  */
 struct option *
 get_opt_rec(struct option *tree, unsigned char *name_)
 {
@@ -241,6 +248,27 @@ get_opt_rec_real(struct option *tree, unsigned char *name)
 	return opt;
 }
 
+/** If @a opt is an alias, return the option to which it refers.
+ *
+ * @warning Because the alias may have the ::OPT_ALIAS_NEGATE flag,
+ * the caller must not access the value of the returned option as if
+ * it were also the value of the alias.  However, it is safe to access
+ * flags such as ::OPT_MUST_SAVE and ::OPT_DELETED.  */
+struct option *
+indirect_option(struct option *alias)
+{
+	struct option *real;
+
+	if (alias->type != OPT_ALIAS) return alias; /* not an error */
+
+	real = get_opt_rec(config_options, alias->value.string);
+	assertm(real != NULL, "%s aliased to unknown option %s!",
+		alias->name, alias->value.string);
+	if_assert_failed return alias;
+
+	return real;
+}
+
 /* Fetch pointer to value of certain option. It is guaranteed to never return
  * NULL. Note that you are supposed to use wrapper get_opt(). */
 union option_value *
@@ -279,11 +307,15 @@ get_opt_(
 		break;
 	case OPT_BOOL:
 	case OPT_INT:
-	case OPT_LONG:
 		if (opt->value.number < opt->min
 		    || opt->value.number > opt->max)
 			elinks_internal("Option %s has invalid value %d!", name, opt->value.number);
 		break;
+	case OPT_LONG:
+		if (opt->value.big_number < opt->min
+		    || opt->value.big_number > opt->max)
+			elinks_internal("Option %s has invalid value %ld!", name, opt->value.big_number);
+		break;
 	case OPT_COMMAND:
 		if (!opt->value.command)
 			elinks_internal("Option %s has no value!", name);
@@ -445,7 +477,7 @@ init_option_listbox_item(struct option *option)
 struct option *
 add_opt(struct option *tree, unsigned char *path, unsigned char *capt,
 	unsigned char *name, enum option_flags flags, enum option_type type,
-	int min, int max, void *value, unsigned char *desc)
+	long min, long max, longptr_T value, unsigned char *desc)
 {
 	struct option *option = mem_calloc(1, sizeof(*option));
 
@@ -465,52 +497,54 @@ add_opt(struct option *tree, unsigned char *path, unsigned char *capt,
 
 	debug_check_option_syntax(option);
 
-	if (option->type != OPT_ALIAS
-	    && ((tree->flags & OPT_LISTBOX) || (option->flags & OPT_LISTBOX))) {
-		option->box_item = init_option_listbox_item(option);
-		if (!option->box_item) {
-			delete_option(option);
-			return NULL;
-		}
-	}
-
 	/* XXX: For allocated values we allocate in the add_opt_<type>() macro.
 	 * This involves OPT_TREE and OPT_STRING. */
 	switch (type) {
 		case OPT_TREE:
 			if (!value) {
-				delete_option(option);
+				mem_free(option);
 				return NULL;
 			}
-			option->value.tree = value;
+			option->value.tree = (struct list_head *) value;
 			break;
 		case OPT_STRING:
 			if (!value) {
-				delete_option(option);
+				mem_free(option);
 				return NULL;
 			}
-			option->value.string = value;
+			option->value.string = (unsigned char *) value;
 			break;
 		case OPT_ALIAS:
-			option->value.string = value;
+			option->value.string = (unsigned char *) value;
 			break;
 		case OPT_BOOL:
 		case OPT_INT:
 		case OPT_CODEPAGE:
+			option->value.number = (int) value;
+			break;
 		case OPT_LONG:
-			option->value.number = (long) value;
+			option->value.big_number = (long) value; /* FIXME: cast from void * */
 			break;
 		case OPT_COLOR:
-			decode_color(value, strlen((unsigned char *) value),
+			decode_color((unsigned char *) value, strlen((unsigned char *) value),
 					&option->value.color);
 			break;
 		case OPT_COMMAND:
-			option->value.command = value;
+			option->value.command = (void *) value;
 			break;
 		case OPT_LANGUAGE:
 			break;
 	}
 
+	if (option->type != OPT_ALIAS
+	    && ((tree->flags & OPT_LISTBOX) || (option->flags & OPT_LISTBOX))) {
+		option->box_item = init_option_listbox_item(option);
+		if (!option->box_item) {
+			mem_free(option);
+			return NULL;
+		}
+	}
+	
 	add_opt_rec(tree, path, option);
 	return option;
 }
@@ -679,6 +713,11 @@ static struct change_hook_info change_hooks[];
 void
 init_options(void)
 {
+	/* The following assignment is needed if void * and struct
+	 * list_head * have different representations.  Otherwise,
+	 * the compiler should be able to optimize it out.  */
+	options_root.value.tree = options_root.value.compile_time_init;
+
 	cmdline_options = add_opt_tree_tree(&options_root, "", "",
 					    "cmdline", 0, "");
 	register_options(cmdline_options_info, cmdline_options);
@@ -724,27 +763,35 @@ register_change_hooks(struct change_hook_info *change_hooks)
 }
 
 void
-unmark_options_tree(struct list_head *tree)
+prepare_mustsave_flags(struct list_head *tree, int set_all)
 {
 	struct option *option;
 
 	foreach (option, *tree) {
-		option->flags &= ~OPT_WATERMARK;
+		/* XXX: OPT_LANGUAGE shouldn't have any bussiness
+		 * here, but we're just weird in that area. */
+		if (set_all
+		    || (option->flags & (OPT_TOUCHED | OPT_DELETED))
+		    || option->type == OPT_LANGUAGE)
+			option->flags |= OPT_MUST_SAVE;
+		else
+			option->flags &= ~OPT_MUST_SAVE;
+
 		if (option->type == OPT_TREE)
-			unmark_options_tree(option->value.tree);
+			prepare_mustsave_flags(option->value.tree, set_all);
 	}
 }
 
 void
-watermark_deleted_options(struct list_head *tree)
+untouch_options(struct list_head *tree)
 {
 	struct option *option;
 
 	foreach (option, *tree) {
-		if (option->flags & OPT_DELETED)
-			option->flags |= OPT_WATERMARK;
-		else if (option->type == OPT_TREE)
-			watermark_deleted_options(option->value.tree);
+		option->flags &= ~OPT_TOUCHED;
+
+		if (option->type == OPT_TREE)
+			untouch_options(option->value.tree);
 	}
 }
 
@@ -757,7 +804,7 @@ check_nonempty_tree(struct list_head *options)
 		if (opt->type == OPT_TREE) {
 			if (check_nonempty_tree(opt->value.tree))
 				return 1;
-		} else if (!(opt->flags & OPT_WATERMARK)) {
+		} else if (opt->flags & OPT_MUST_SAVE) {
 			return 1;
 		}
 	}
@@ -777,14 +824,14 @@ smart_config_string(struct string *str, int print_comment, int i18n,
 		int do_print_comment = 1;
 
 		if (option->flags & OPT_HIDDEN ||
-		    option->flags & OPT_WATERMARK ||
 		    option->type == OPT_ALIAS ||
 		    !strcmp(option->name, "_template_"))
 			continue;
 
 		/* Is there anything to be printed anyway? */
 		if (option->type == OPT_TREE
-		    && !check_nonempty_tree(option->value.tree))
+		    ? !check_nonempty_tree(option->value.tree)
+		    : !(option->flags & OPT_MUST_SAVE))
 			continue;
 
 		/* We won't pop out the description when we're in autocreate
@@ -850,10 +897,6 @@ smart_config_string(struct string *str, int print_comment, int i18n,
 
 			fn(str, option, path, depth, /*pc*/1, 3, i18n);
 		}
-
-		/* TODO: We should maybe clear the touched flag only when really
-		 * saving the stuff...? --pasky */
-		option->flags &= ~OPT_TOUCHED;
 	}
 }
 
@@ -1106,26 +1149,42 @@ register_options(struct option_info info[], struct option *tree)
 					delete_option(option);
 					continue;
 				}
-				safe_strncpy(string, option->value.string, MAX_STR_LEN);
+				safe_strncpy(string,
+					     option->value.compile_time_init,
+					     MAX_STR_LEN);
 				option->value.string = string;
 				break;
 			case OPT_COLOR:
-				string = option->value.string;
+				string = option->value.compile_time_init;
 				assert(string);
 				decode_color(string, strlen(string),
 						&option->value.color);
 				break;
 			case OPT_CODEPAGE:
-				string = option->value.string;
+				string = option->value.compile_time_init;
 				assert(string);
 				option->value.number = get_cp_index(string);
 				break;
 			case OPT_BOOL:
 			case OPT_INT:
-			case OPT_LONG:
 			case OPT_LANGUAGE:
+				/* Some compilers warn if we cast a
+				 * pointer directly to an integer with
+				 * a different size.  */
+				option->value.number = (int) (longptr_T)
+					option->value.compile_time_init;
+				break;
+			case OPT_LONG:
+				option->value.big_number = (long) (longptr_T)
+					option->value.compile_time_init;
+				break;
 			case OPT_COMMAND:
+				option->value.command =
+					option->value.compile_time_init;
+				break;
 			case OPT_ALIAS:
+				option->value.string =
+					option->value.compile_time_init;
 				break;
 		}
 
