@@ -82,7 +82,7 @@ get_target(struct document_options *options, unsigned char *a)
 
 	if (!v) return NULL;
 
-	if (!*v || !strcasecmp(v, "_self")) {
+	if (!*v || !c_strcasecmp(v, "_self")) {
 		mem_free_set(&v, stracpy(options->framename));
 	}
 
@@ -262,6 +262,36 @@ html_skip(struct html_context *html_context, unsigned char *a)
 	html_top.type = ELEMENT_DONT_KILL;
 }
 
+/* Parse meta refresh without URL= in it:
+ *  <meta http-equiv="refresh" content="3,http://elinks.or.cz/">
+ *  <meta http-equiv="refresh" content="3; http://elinks.or.cz/">
+ *  <meta http-equiv="refresh" content="   3 ;   http://elinks.or.cz/    ">
+ */
+static void
+parse_old_meta_refresh(unsigned char *str, unsigned char **ret)
+{
+	unsigned char *p = str;
+	int len;
+
+	assert(str && ret);
+	if_assert_failed return;
+
+	*ret = NULL;
+	while (*p && (*p == ' ' || *p == ASCII_TAB)) p++;
+	if (!*p) return;
+	while (*p && *p >= '0' && *p <= '9') p++;
+	if (!*p) return;
+	while (*p && (*p == ' ' || *p == ASCII_TAB)) p++;
+	if (!*p) return;
+	if (*p == ';' || *p == ',') p++; else return;
+	while (*p && (*p == ' ' || *p == ASCII_TAB)) p++;
+	if (!*p) return;
+
+	len = strlen(p);
+	while (len && (p[len] == ' ' || p[len] == ASCII_TAB)) len--;
+	if (len) *ret = memacpy(p, len);
+}
+
 void
 process_head(struct html_context *html_context, unsigned char *head)
 {
@@ -272,9 +302,13 @@ process_head(struct html_context *html_context, unsigned char *head)
 
 	url = parse_header_param(refresh, "URL");
 	if (!url) {
-		/* If the URL parameter is missing assume that the
-		 * document being processed should be refreshed. */
-		url = get_uri_string(html_context->base_href, URI_ORIGINAL);
+		/* Let's try a more tolerant parsing. */
+		parse_old_meta_refresh(refresh, &url);
+		if (!url) {
+			/* If the URL parameter is missing assume that the
+			 * document being processed should be refreshed. */
+			url = get_uri_string(html_context->base_href, URI_ORIGINAL);
+		}
 	}
 
 	if (url) {
@@ -411,13 +445,13 @@ look_for_map(unsigned char **pos, unsigned char *eof, struct uri *uri,
 		return 1;
 	}
 
-	if (strlcasecmp(name, namelen, "MAP", 3)) return 1;
+	if (c_strlcasecmp(name, namelen, "MAP", 3)) return 1;
 
 	if (uri && uri->fragment) {
 		al = get_attr_val(attr, "name", options);
 		if (!al) return 1;
 
-		if (strlcasecmp(al, -1, uri->fragment, uri->fragmentlen)) {
+		if (c_strlcasecmp(al, -1, uri->fragment, uri->fragmentlen)) {
 			mem_free(al);
 			return 1;
 		}
@@ -464,12 +498,12 @@ look_for_tag(unsigned char **pos, unsigned char *eof,
 
 	if (parse_element(*pos, eof, NULL, NULL, NULL, &pos2)) return 1;
 
-	if (strlcasecmp(name, namelen, "A", 1)
-	    && strlcasecmp(name, namelen, "/A", 2)
-	    && strlcasecmp(name, namelen, "MAP", 3)
-	    && strlcasecmp(name, namelen, "/MAP", 4)
-	    && strlcasecmp(name, namelen, "AREA", 4)
-	    && strlcasecmp(name, namelen, "/AREA", 5)) {
+	if (c_strlcasecmp(name, namelen, "A", 1)
+	    && c_strlcasecmp(name, namelen, "/A", 2)
+	    && c_strlcasecmp(name, namelen, "MAP", 3)
+	    && c_strlcasecmp(name, namelen, "/MAP", 4)
+	    && c_strlcasecmp(name, namelen, "AREA", 4)
+	    && c_strlcasecmp(name, namelen, "/AREA", 5)) {
 		*pos = pos2;
 		return 1;
 	}
@@ -477,6 +511,9 @@ look_for_tag(unsigned char **pos, unsigned char *eof,
 	return 0;
 }
 
+/** @return -1 if EOF is hit without the closing tag; 0 if the closing
+ * tag is found (in which case this also adds *@a menu to *@a ml); or
+ * 1 if this should be called again.  */
 static int
 look_for_link(unsigned char **pos, unsigned char *eof, struct menu_item **menu,
 	      struct memory_list **ml, struct uri *href_base,
@@ -494,7 +531,7 @@ look_for_link(unsigned char **pos, unsigned char *eof, struct menu_item **menu,
 		(*pos)++;
 	}
 
-	if (*pos >= eof) return 0;
+	if (*pos >= eof) return -1;
 
 	if (*pos + 2 <= eof && ((*pos)[1] == '!' || (*pos)[1] == '?')) {
 		*pos = skip_comment(*pos, eof);
@@ -506,12 +543,12 @@ look_for_link(unsigned char **pos, unsigned char *eof, struct menu_item **menu,
 		return 1;
 	}
 
-	if (!strlcasecmp(name, namelen, "A", 1)) {
+	if (!c_strlcasecmp(name, namelen, "A", 1)) {
 		while (look_for_tag(pos, eof, name, namelen, &label));
 
-		if (*pos >= eof) return 0;
+		if (*pos >= eof) return -1;
 
-	} else if (!strlcasecmp(name, namelen, "AREA", 4)) {
+	} else if (!c_strlcasecmp(name, namelen, "AREA", 4)) {
 		unsigned char *alt = get_attr_val(attr, "alt", options);
 
 		if (alt) {
@@ -523,7 +560,7 @@ look_for_link(unsigned char **pos, unsigned char *eof, struct menu_item **menu,
 			label = NULL;
 		}
 
-	} else if (!strlcasecmp(name, namelen, "/MAP", 4)) {
+	} else if (!c_strlcasecmp(name, namelen, "/MAP", 4)) {
 		/* This is the only successful return from here! */
 		add_to_ml(ml, *menu, NULL);
 		return 0;
@@ -533,8 +570,7 @@ look_for_link(unsigned char **pos, unsigned char *eof, struct menu_item **menu,
 	}
 
 	target = get_target(options, attr);
-	if (!target) target = null_or_stracpy(target_base);
-	if (!target) target = stracpy("");
+	if (!target) target = stracpy(empty_string_or_(target_base));
 	if (!target) {
 		mem_free_if(label);
 		return 1;
@@ -623,6 +659,7 @@ get_image_map(unsigned char *head, unsigned char *pos, unsigned char *eof,
 {
 	struct conv_table *ct;
 	struct string hd;
+	int look_result;
 
 	if (!init_string(&hd)) return -1;
 
@@ -643,10 +680,13 @@ get_image_map(unsigned char *head, unsigned char *pos, unsigned char *eof,
 
 	*ml = NULL;
 
-	while (look_for_link(&pos, eof, menu, ml, uri, target_base, ct, options))
-		;
+	do {
+		/* This call can modify both *ml and *menu.  */
+		look_result = look_for_link(&pos, eof, menu, ml, uri,
+					    target_base, ct, options);
+	} while (look_result > 0);
 
-	if (pos >= eof) {
+	if (look_result < 0) {
 		freeml(*ml);
 		mem_free(*menu);
 		return -1;
@@ -660,7 +700,7 @@ get_image_map(unsigned char *head, unsigned char *pos, unsigned char *eof,
 
 struct html_element *
 init_html_parser_state(struct html_context *html_context,
-                       enum html_element_type type,
+                       enum html_element_mortality_type type,
                        int align, int margin, int width)
 {
 	struct html_element *element;
