@@ -282,13 +282,13 @@ parse_optional_fields(struct mailcap_entry *entry, unsigned char *line)
 
 		if (!field) break;
 
-		if (!strncasecmp(field, "needsterminal", 13)) {
+		if (!c_strncasecmp(field, "needsterminal", 13)) {
 				entry->needsterminal = 1;
 
-		} else if (!strncasecmp(field, "copiousoutput", 13)) {
+		} else if (!c_strncasecmp(field, "copiousoutput", 13)) {
 			entry->copiousoutput = 1;
 
-		} else if (!strncasecmp(field, "test", 4)) {
+		} else if (!c_strncasecmp(field, "test", 4)) {
 			entry->testcommand = get_mailcap_field_text(field + 4);
 			if (!entry->testcommand)
 				return 0;
@@ -300,7 +300,7 @@ parse_optional_fields(struct mailcap_entry *entry, unsigned char *line)
 					return 0;
 				}
 
-		} else if (!strncasecmp(field, "description", 11)) {
+		} else if (!c_strncasecmp(field, "description", 11)) {
 			entry->description = get_mailcap_field_text(field + 11);
 			if (!entry->description)
 				return 0;
