@@ -64,6 +64,16 @@ struct connect_info {
 	int ip_family;			 /* If non-zero, force to IP version. */
 };
 
+/** For detecting whether a struct socket has been deleted while a
+ * function was using it.  */
+struct socket_weak_ref {
+	LIST_HEAD(struct socket_weak_ref);
+
+	/** done_socket() resets this to NULL.  */
+	struct socket *socket;
+};
+
+static INIT_LIST_HEAD(socket_weak_refs); /* -> struct socket_weak_ref */
 
 /* To enable logging of tranfers, for debugging purposes. */
 #if 0
@@ -136,6 +146,8 @@ init_socket(void *conn, struct socket_operations *ops)
 void
 done_socket(struct socket *socket)
 {
+	struct socket_weak_ref *ref;
+
 	close_socket(socket);
 
 	if (socket->connect_info)
@@ -143,6 +155,11 @@ done_socket(struct socket *socket)
 
 	mem_free_set(&socket->read_buffer, NULL);
 	mem_free_set(&socket->write_buffer, NULL);
+
+	foreach(ref, socket_weak_refs) {
+		if (ref->socket == socket)
+			ref->socket = NULL;
+	}
 }
 
 void
@@ -279,6 +296,7 @@ get_pasv_socket(struct socket *ctrl_socket, struct sockaddr_storage *addr)
 	struct sockaddr *pasv_addr = (struct sockaddr *) addr;
 	size_t addrlen;
 	int sock = -1;
+	int syspf; /* Protocol Family given to system, not EL_PF_... */
 	socklen_t len;
 #ifdef CONFIG_IPV6
 	struct sockaddr_in6 bind_addr6;
@@ -286,15 +304,17 @@ get_pasv_socket(struct socket *ctrl_socket, struct sockaddr_storage *addr)
 	if (ctrl_socket->protocol_family == EL_PF_INET6) {
 		bind_addr = (struct sockaddr *) &bind_addr6;
 		addrlen   = sizeof(bind_addr6);
+		syspf     = PF_INET6;
 	} else
 #endif
 	{
 		bind_addr = (struct sockaddr *) &bind_addr4;
 		addrlen   = sizeof(bind_addr4);
+		syspf     = PF_INET;
 	}
 
-	memset(pasv_addr, 0, sizeof(addrlen));
-	memset(bind_addr, 0, sizeof(addrlen));
+	memset(pasv_addr, 0, addrlen);
+	memset(bind_addr, 0, addrlen);
 
 	/* Get our endpoint of the control socket */
 	len = addrlen;
@@ -307,7 +327,7 @@ sock_error:
 
 	/* Get a passive socket */
 
-	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+	sock = socket(syspf, SOCK_STREAM, IPPROTO_TCP);
 	if (sock < 0)
 		goto sock_error;
 
@@ -583,23 +603,24 @@ connect_socket(struct socket *csocket, enum connection_state state)
 		 * will fail, as we will use it only when it will be successfully
 		 * established. At least I hope that noone else will want to do
 		 * something else ;-). --pasky */
+		/* And in fact we must set it early, because of EINPROGRESS.  */
 
 #ifdef CONFIG_IPV6
 		if (addr.sin6_family == AF_INET6) {
+			csocket->protocol_family = EL_PF_INET6;
 			if (connect(sock, (struct sockaddr *) &addr,
 					sizeof(struct sockaddr_in6)) == 0) {
 				/* Success */
-				csocket->protocol_family = EL_PF_INET6;
 				complete_connect_socket(csocket, NULL, NULL);
 				return;
 			}
 		} else
 #endif
 		{
+			csocket->protocol_family = EL_PF_INET;
 			if (connect(sock, (struct sockaddr *) &addr,
 					sizeof(struct sockaddr_in)) == 0) {
 				/* Success */
-				csocket->protocol_family = EL_PF_INET;
 				complete_connect_socket(csocket, NULL, NULL);
 				return;
 			}
@@ -900,13 +921,26 @@ void
 read_from_socket(struct socket *socket, struct read_buffer *buffer,
 		 enum connection_state state, socket_read_T done)
 {
+	const int is_buffer_new = (buffer != socket->read_buffer);
+	struct socket_weak_ref ref;
 	select_handler_T write_handler;
 
+	ref.socket = socket;
+	add_to_list(socket_weak_refs, &ref);
+
 	buffer->done = done;
 
 	socket->ops->set_timeout(socket, 0);
 	socket->ops->set_state(socket, state);
 
+	del_from_list(&ref);
+	if (ref.socket == NULL) {
+		/* socket->ops->set_state deleted the socket. */
+		if (is_buffer_new)
+			mem_free(buffer);
+		return;
+	}
+
 	if (socket->read_buffer && buffer != socket->read_buffer)
 		mem_free(socket->read_buffer);
 	socket->read_buffer = buffer;
