@@ -190,13 +190,13 @@ static struct option_info http_options[] = {
 		"is sent to HTTP server when a document is requested. The 'textmode'\n"
 		"token in the first field is our silent attempt to establish this as\n"
 		"a standard for new textmode user agents, so that the webmasters can\n"
-		"have just a single uniform test for these if they are ie. pushing\n"
+		"have just a single uniform test for these if they are e.g. pushing\n"
 		"some lite version to them automagically.\n"
-		"%v in the string means ELinks version\n"
-		"%s in the string means system identification\n"
-		"%t in the string means size of the terminal\n"
-		"%b in the string means number of bars displayed by ELinks\n"
-		"Use \" \" if you don't want any User-Agent header to be sent at all.")),
+		"Use \" \" if you don't want any User-Agent header to be sent at all.\n"
+		"%v in the string means ELinks version,\n"
+		"%s in the string means system identification,\n"
+		"%t in the string means size of the terminal,\n"
+		"%b in the string means number of bars displayed by ELinks.")),
 
 
 	INIT_OPT_TREE("protocol", N_("HTTPS"),
@@ -390,8 +390,8 @@ get_http_code(struct read_buffer *rb, int *code, struct http_version *version)
 	while (*head == ' ') head++;
 
 	/* HTTP/ */
-	if (toupper(*head) != 'H' || toupper(*++head) != 'T' ||
-	    toupper(*++head) != 'T' || toupper(*++head) != 'P'
+	if (c_toupper(*head) != 'H' || c_toupper(*++head) != 'T' ||
+	    c_toupper(*++head) != 'T' || c_toupper(*++head) != 'P'
 	    || *++head != '/')
 		return -1;
 
@@ -551,7 +551,7 @@ http_send_header(struct socket *socket)
 	int trace = get_opt_bool("protocol.http.trace");
 	struct string header;
 	unsigned char *post_data = NULL;
-	struct auth_entry *entry;
+	struct auth_entry *entry = NULL;
 	struct uri *uri = conn->proxied_uri; /* Set to the real uri */
 	unsigned char *optstr;
 	int use_connect, talking_to_proxy;
@@ -577,6 +577,11 @@ http_send_header(struct socket *socket)
 		add_to_string(&header, "TRACE ");
 	} else if (use_connect) {
 		add_to_string(&header, "CONNECT ");
+		/* In CONNECT requests, we send only a subset of the
+		 * headers to the proxy.  See the "CONNECT:" comments
+		 * below.  After the CONNECT request succeeds, we
+		 * negotiate TLS with the real server and make a new
+		 * HTTP request that includes all the headers.  */
 	} else if (uri->post) {
 		add_to_string(&header, "POST ");
 		conn->unrestartable = 1;
@@ -609,10 +614,14 @@ http_send_header(struct socket *socket)
 	add_long_to_string(&header, http->sent_version.minor);
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Sending a Host header seems pointless as the same
+	 * information is already in the CONNECT line.  It's harmless
+	 * though and Mozilla does it too.  */
 	add_to_string(&header, "Host: ");
 	add_uri_to_string(&header, uri, URI_HTTP_HOST);
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Proxy-Authorization is intended to be seen by the proxy.  */
 	if (talking_to_proxy) {
 		unsigned char *user = get_opt_str("protocol.http.proxy.user");
 		unsigned char *passwd = get_opt_str("protocol.http.proxy.passwd");
@@ -660,6 +669,9 @@ http_send_header(struct socket *socket)
 		}
 	}
 
+	/* CONNECT: User-Agent does not reveal anything about the
+	 * resource we're fetching, and it may help the proxy return
+	 * better error messages.  */
 	optstr = get_opt_str("protocol.http.user_agent");
 	if (*optstr && strcmp(optstr, " ")) {
 		unsigned char *ustr, ts[64] = "";
@@ -685,33 +697,47 @@ http_send_header(struct socket *socket)
 		add_crlf_to_string(&header);
 	}
 
-	switch (get_opt_int("protocol.http.referer.policy")) {
-		case REFERER_NONE:
-			/* oh well */
-			break;
+	/* CONNECT: Referer probably is a secret page in the HTTPS
+	 * server, so don't reveal it to the proxy.  */ 
+	if (!use_connect) {
+		switch (get_opt_int("protocol.http.referer.policy")) {
+			case REFERER_NONE:
+				/* oh well */
+				break;
 
-		case REFERER_FAKE:
-			optstr = get_opt_str("protocol.http.referer.fake");
-			if (!optstr[0]) break;
-			add_to_string(&header, "Referer: ");
-			add_to_string(&header, optstr);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_FAKE:
+				optstr = get_opt_str("protocol.http.referer.fake");
+				if (!optstr[0]) break;
+				add_to_string(&header, "Referer: ");
+				add_to_string(&header, optstr);
+				add_crlf_to_string(&header);
+				break;
 
-		case REFERER_TRUE:
-			if (!conn->referrer) break;
-			add_to_string(&header, "Referer: ");
-			add_url_to_http_string(&header, conn->referrer, URI_HTTP_REFERRER);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_TRUE:
+				if (!conn->referrer) break;
+				add_to_string(&header, "Referer: ");
+				add_url_to_http_string(&header, conn->referrer, URI_HTTP_REFERRER);
+				add_crlf_to_string(&header);
+				break;
 
-		case REFERER_SAME_URL:
-			add_to_string(&header, "Referer: ");
-			add_url_to_http_string(&header, uri, URI_HTTP_REFERRER);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_SAME_URL:
+				add_to_string(&header, "Referer: ");
+				add_url_to_http_string(&header, uri, URI_HTTP_REFERRER);
+				add_crlf_to_string(&header);
+				break;
+		}
 	}
 
+	/* CONNECT: Do send all Accept* headers to the CONNECT proxy,
+	 * because they do not reveal anything about the resource
+	 * we're going to request via TLS, and they may affect the
+	 * error message if the CONNECT request fails.
+	 *
+	 * If ELinks is ever changed to vary its Accept headers based
+	 * on what it intends to do with the returned resource, e.g.
+	 * sending "Accept: text/css" when it wants an external
+	 * stylesheet, then it should do that only in the inner GET
+	 * and not in the outer CONNECT.  */
 	add_to_string(&header, "Accept: */*");
 	add_crlf_to_string(&header);
 
@@ -766,6 +792,11 @@ http_send_header(struct socket *socket)
 	}
 #endif
 
+	/* CONNECT: Proxy-Connection is intended to be seen by the
+	 * proxy.  If the CONNECT request succeeds, then the proxy
+	 * will forward the remainder of the TCP connection to the
+	 * origin server, and Proxy-Connection does not matter; but
+	 * if the request fails, then Proxy-Connection may matter.  */
 	/* FIXME: What about post-HTTP/1.1?? --Zas */
 	if (HTTP_1_1(http->sent_version)) {
 		if (!IS_PROXY_URI(conn->uri)) {
@@ -782,7 +813,9 @@ http_send_header(struct socket *socket)
 		add_crlf_to_string(&header);
 	}
 
-	if (conn->cached) {
+ 	/* CONNECT: Do not tell the proxy anything we have cached
+ 	 * about the resource.  */
+ 	if (!use_connect && conn->cached) {
 		if (!conn->cached->incomplete && conn->cached->head && conn->cached->last_modified
 		    && conn->cache_mode <= CACHE_MODE_CHECK_IF_MODIFIED) {
 			add_to_string(&header, "If-Modified-Since: ");
@@ -791,6 +824,8 @@ http_send_header(struct socket *socket)
 		}
 	}
 
+	/* CONNECT: Let's send cache control headers to the proxy too;
+	 * they may affect DNS caching.  */
 	if (conn->cache_mode >= CACHE_MODE_FORCE_RELOAD) {
 		add_to_string(&header, "Pragma: no-cache");
 		add_crlf_to_string(&header);
@@ -798,7 +833,9 @@ http_send_header(struct socket *socket)
 		add_crlf_to_string(&header);
 	}
 
-	if (conn->from || conn->progress->start > 0) {
+	/* CONNECT: Do not reveal byte ranges to the proxy.  It can't
+	 * do anything good with that information anyway.  */
+	if (!use_connect && (conn->from || conn->progress->start > 0)) {
 		/* conn->from takes precedence. conn->progress.start is set only the first
 		 * time, then conn->from gets updated and in case of any retries
 		 * etc we have everything interesting in conn->from already. */
@@ -808,7 +845,10 @@ http_send_header(struct socket *socket)
 		add_crlf_to_string(&header);
 	}
 
-	entry = find_auth(uri);
+ 	/* CONNECT: The Authorization header is for the origin server only.  */
+ 	if (!use_connect) {
+		entry = find_auth(uri);
+ 	}
 	if (entry) {
 		if (entry->digest) {
 			unsigned char *response;
@@ -848,7 +888,8 @@ http_send_header(struct socket *socket)
 		}
 	}
 
-	if (uri->post) {
+	/* CONNECT: Any POST data is for the origin server only.  */
+	if (!use_connect && uri->post) {
 		/* We search for first '\n' in uri->post to get content type
 		 * as set by get_form_uri(). This '\n' is dropped if any
 		 * and replaced by correct '\r\n' termination here. */
@@ -867,7 +908,8 @@ http_send_header(struct socket *socket)
 	}
 
 #ifdef CONFIG_COOKIES
-	{
+	/* CONNECT: Cookies are for the origin server only.  */
+	if (!use_connect) {
 		struct string *cookies = send_cookies(uri);
 
 		if (cookies) {
@@ -881,12 +923,17 @@ http_send_header(struct socket *socket)
 
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Any POST data is for the origin server only.
+	 * This was already checked above and post_data is NULL
+	 * in that case.  Verified with an assertion below.  */
 	if (post_data) {
 #define POST_BUFFER_SIZE 4096
 		unsigned char *post = post_data;
 		unsigned char buffer[POST_BUFFER_SIZE];
 		int n = 0;
 
+		assert(!use_connect); /* see comment above */
+
 		while (post[0] && post[1]) {
 			int h1, h2;
 
@@ -1300,7 +1347,7 @@ get_header(struct read_buffer *rb)
 	/* XXX: We will have to do some guess about whether an HTTP header is
 	 * coming or not, in order to support HTTP/0.9 reply correctly. This
 	 * means a little code duplication with get_http_code(). --pasky */
-	if (rb->length > 4 && strncasecmp(rb->data, "HTTP/", 5))
+	if (rb->length > 4 && c_strncasecmp(rb->data, "HTTP/", 5))
 		return -2;
 
 	for (i = 0; i < rb->length; i++) {
@@ -1336,7 +1383,7 @@ check_http_authentication(struct uri *uri, unsigned char *header,
 
 	d = parse_header(header, header_field, &str);
 	while (d) {
-		if (!strncasecmp(d, "Basic", 5)) {
+		if (!c_strncasecmp(d, "Basic", 5)) {
 			unsigned char *realm = get_header_param(d, "realm");
 
 			if (realm) {
@@ -1345,7 +1392,7 @@ check_http_authentication(struct uri *uri, unsigned char *header,
 				mem_free(d);
 				break;
 			}
-		} else if (!strncasecmp(d, "Digest", 6)) {
+		} else if (!c_strncasecmp(d, "Digest", 6)) {
 			unsigned char *realm = get_header_param(d, "realm");
 			unsigned char *nonce = get_header_param(d, "nonce");
 			unsigned char *opaque = get_header_param(d, "opaque");
@@ -1376,7 +1423,7 @@ http_got_header(struct socket *socket, struct read_buffer *rb)
 #endif
 	unsigned char *d;
 	struct uri *uri = conn->proxied_uri; /* Set to the real uri */
-	struct http_version version;
+	struct http_version version = { 0, 9 };
 	enum connection_state state = (conn->state != S_PROC ? S_GETH : S_PROC);
 	int a, h = 200;
 	int cf;
@@ -1405,6 +1452,9 @@ again:
 		read_from_socket(conn->socket, rb, state, http_got_header);
 		return;
 	}
+	/* a == -2 from get_header means HTTP/0.9.  In that case, skip
+	 * the get_http_code call; @h and @version have already been
+	 * initialized with the right values.  */
 	if (a == -2) a = 0;
 	if ((a && get_http_code(rb, &h, &version))
 	    || h == 101) {
@@ -1598,7 +1648,7 @@ again:
 
 		d = parse_header(conn->cached->head, "Proxy-Authenticate", &str);
 		while (d) {
-			if (!strncasecmp(d, "Basic", 5)) {
+			if (!c_strncasecmp(d, "Basic", 5)) {
 				unsigned char *realm = get_header_param(d, "realm");
 
 				if (realm) {
@@ -1608,7 +1658,7 @@ again:
 					break;
 				}
 
-			} else if (!strncasecmp(d, "Digest", 6)) {
+			} else if (!c_strncasecmp(d, "Digest", 6)) {
 				unsigned char *realm = get_header_param(d, "realm");
 				unsigned char *nonce = get_header_param(d, "nonce");
 				unsigned char *opaque = get_header_param(d, "opaque");
@@ -1634,7 +1684,7 @@ again:
 
 	if ((d = parse_header(conn->cached->head, "Connection", NULL))
 	     || (d = parse_header(conn->cached->head, "Proxy-Connection", NULL))) {
-		if (!strcasecmp(d, "close")) http->close = 1;
+		if (!c_strcasecmp(d, "close")) http->close = 1;
 		mem_free(d);
 	} else if (PRE_HTTP_1_1(version)) {
 		http->close = 1;
@@ -1646,7 +1696,7 @@ again:
 	if (d) {
 		if (strlen(d) > 6) {
 			d[5] = 0;
-			if (isdigit(d[6]) && !strcasecmp(d, "bytes")) {
+			if (isdigit(d[6]) && !c_strcasecmp(d, "bytes")) {
 				int f;
 
 				errno = 0;
@@ -1702,7 +1752,7 @@ again:
 		d = parse_header(conn->cached->head, "Accept-Ranges", NULL);
 
 		if (d) {
-			if (!strcasecmp(d, "none"))
+			if (!c_strcasecmp(d, "none"))
 				conn->unrestartable = 1;
 			mem_free(d);
 		} else {
@@ -1713,7 +1763,7 @@ again:
 
 	d = parse_header(conn->cached->head, "Transfer-Encoding", NULL);
 	if (d) {
-		if (!strcasecmp(d, "chunked")) {
+		if (!c_strcasecmp(d, "chunked")) {
 			http->length = LEN_CHUNKED;
 			http->chunk_remaining = CHUNK_SIZE;
 		}
@@ -1723,7 +1773,7 @@ again:
 
 	d = parse_header(conn->cached->head, "Last-Modified", NULL);
 	if (d) {
-		if (conn->cached->last_modified && strcasecmp(conn->cached->last_modified, d)) {
+		if (conn->cached->last_modified && c_strcasecmp(conn->cached->last_modified, d)) {
 			delete_entry_content(conn->cached);
 			if (conn->from) {
 				conn->from = 0;
@@ -1785,13 +1835,13 @@ again:
 		 * will leave the saved file with the correct encoding. */
 #ifdef CONFIG_GZIP
 		if (file_encoding != ENCODING_GZIP
-		    && (!strcasecmp(d, "gzip") || !strcasecmp(d, "x-gzip")))
+		    && (!c_strcasecmp(d, "gzip") || !c_strcasecmp(d, "x-gzip")))
 		    	conn->content_encoding = ENCODING_GZIP;
 #endif
 #ifdef BUG_517
 #ifdef CONFIG_BZIP2
 		if (file_encoding != ENCODING_BZIP2
-		    && (!strcasecmp(d, "bzip2") || !strcasecmp(d, "x-bzip2")))
+		    && (!c_strcasecmp(d, "bzip2") || !c_strcasecmp(d, "x-bzip2")))
 			conn->content_encoding = ENCODING_BZIP2;
 #endif
 #endif
