@@ -108,7 +108,7 @@ end_with_known_tld(unsigned char *s, int slen)
 		int tldlen = strlen(tld[i]);
 		int pos = slen - tldlen;
 
-		if (pos >= 0 && !strncasecmp(&s[pos], tld[i], tldlen))
+		if (pos >= 0 && !c_strncasecmp(&s[pos], tld[i], tldlen))
 			return pos;
 	}
 
@@ -149,7 +149,7 @@ static int
 check_uri_file(unsigned char *name)
 {
 	/* Check POST_CHAR etc ... */
-	static const unsigned char chars[] = POST_CHAR_S "#?";
+	static const unsigned char chars[] = POST_CHAR_S "#";
 
 	return strcspn(name, chars);
 }
@@ -161,6 +161,7 @@ encode_file_uri_string(struct string *string, unsigned char *uristring)
 	int filenamelen = check_whether_file_exists(uristring);
 
 	encode_uri_string(string, uristring, filenamelen, 0);
+	if (filenamelen > 0) add_to_string(string, uristring + filenamelen);
 }
 
 
@@ -681,7 +682,7 @@ normalize_uri(struct uri *uri, unsigned char *uristring)
 		 * get_translated_uri() through translate_url() calls this
 		 * function and then it already works on and modifies an
 		 * allocated copy. */
-		convert_to_lowercase(uri->string, uri->protocollen);
+		convert_to_lowercase_locale_indep(uri->string, uri->protocollen);
 		if (uri->hostlen) convert_to_lowercase(uri->host, uri->hostlen);
 
 		parse = 1;
@@ -988,7 +989,7 @@ find_uri_protocol(unsigned char *newurl)
 	ch = newurl + strcspn(newurl, ".:/@");
 	if (*ch == '@'
 	    || (*ch == ':' && *newurl != '[' && strchr(newurl, '@'))
-	    || !strncasecmp(newurl, "ftp.", 4)) {
+	    || !c_strncasecmp(newurl, "ftp.", 4)) {
 		/* Contains user/password/ftp-hostname */
 		return PROTOCOL_FTP;
 
@@ -1463,11 +1464,11 @@ check_uri_sanity(struct uri *uri)
 	int pos;
 
 	for (pos = 0; pos < uri->protocollen; pos++)
-		if (isupper(uri->string[pos])) goto error;
+		if (c_isupper(uri->string[pos])) goto error;
 
 	if (uri->hostlen)
 		for (pos = 0; pos < uri->hostlen; pos++)
-			if (isupper(uri->host[pos])) goto error;
+			if (c_isupper(uri->host[pos])) goto error;
 	return;
 error:
 	INTERNAL("Uppercase letters detected in protocol or host part (%s).", struri(uri));
