@@ -70,7 +70,13 @@ static void handle_ref(LS, struct session *ses, int func_ref,
 static int
 l_alert(LS)
 {
-	alert_lua_error((unsigned char *) lua_tostring(S, 1));
+	unsigned char *msg = (unsigned char *) lua_tostring(S, 1);
+
+	/* Don't crash if a script calls e.g. error(nil) or error(error).  */
+	if (msg == NULL)
+		msg = "(cannot convert the error message to a string)";
+
+	alert_lua_error(msg);
 	return 0;
 }
 
@@ -197,8 +203,10 @@ l_pipe_read(LS)
 		size_t l = fread(buf, 1, sizeof(buf), fp);
 
 		if (l > 0) {
-			s = mem_realloc(s, len + l);
-			if (!s) goto lua_error;
+			unsigned char *news = mem_realloc(s, len + l);
+
+			if (!news) goto lua_error;
+			s = news;
 			memcpy(s + len, buf, l);
 			len += l;
 
@@ -494,19 +502,21 @@ l_set_option(LS)
 	switch (opt->type) {
 	case OPT_BOOL:
 	{
-		int value;
+		/* option_types[OPT_BOOL].set expects a long even though it
+		 * saves the value to opt->value.number, which is an int.  */
+		long value = lua_toboolean(S, 2);
 
-		value = lua_toboolean(S, 2);
 		option_types[opt->type].set(opt, (unsigned char *) (&value));
 		break;
 	}
-
 	case OPT_INT:
 	case OPT_LONG:
 	{
-		int value;
+		/* option_types[OPT_INT].set expects a long even though it
+		 * saves the value to opt->value.number, which is an int.
+		 * option_types[OPT_LONG].set of course wants a long too.  */
+		long value = lua_tonumber(S, 2);
 
-		value = lua_tonumber(S, 2);
 		option_types[opt->type].set(opt, (unsigned char *) (&value));
 		break;
 	}
@@ -554,9 +564,11 @@ l_get_option(LS)
 		lua_pushboolean(S, opt->value.number);
 		break;
 	case OPT_INT:
-	case OPT_LONG:
 		lua_pushnumber(S, opt->value.number);
 		break;
+	case OPT_LONG:
+		lua_pushnumber(S, opt->value.big_number);
+		break;
 	case OPT_STRING:
 		lua_pushstring(S, opt->value.string);
 		break;
