@@ -29,14 +29,15 @@ keymap_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &keymap_class, NULL))
 		return JS_FALSE;
 
-	data = JS_GetPrivate(ctx, obj); /* from @keymap_class */
+	data = JS_GetInstancePrivate(ctx, obj,
+				     (JSClass *) &keymap_class, NULL);
 
 	keystroke_str = JS_GetStringBytes(JS_ValueToString(ctx, id));
 	if (!keystroke_str) goto ret_null;
 
 	action_str = get_action_name_from_keystroke((enum keymap_id) *data,
 						    keystroke_str);
-	if (!action_str) goto ret_null;
+	if (!action_str || !strcmp(action_str, "none")) goto ret_null;
 
 	*vp = STRING_TO_JSVAL(JS_NewStringCopyZ(ctx, action_str));
 
@@ -83,7 +84,8 @@ keymap_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &keymap_class, NULL))
 		return JS_FALSE;
 
-	data = JS_GetPrivate(ctx, obj); /* from @keymap_class */
+	data = JS_GetInstancePrivate(ctx, obj,
+				     (JSClass *) &keymap_class, NULL);
 
 	/* Ugly fact: we need to get the string from the id to give to bind_do,
 	 * which will of course then convert the string back to an id... */
@@ -104,6 +106,12 @@ keymap_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 
 		return JS_TRUE;
 
+	} else if (JSVAL_IS_NULL(*vp)) { /* before JSVAL_IS_OBJECT */
+		if (bind_do(keymap_str, keystroke_str, "none", 0))
+			return JS_FALSE;
+
+		return JS_TRUE;
+
 	} else if (JSVAL_IS_OBJECT(*vp)) {
 		unsigned char *err = NULL;
 		int event_id;
@@ -155,7 +163,8 @@ keymap_finalize(JSContext *ctx, JSObject *obj)
 	assert(JS_InstanceOf(ctx, obj, (JSClass *) &keymap_class, NULL));
 	if_assert_failed return;
 
-	data = JS_GetPrivate(ctx, obj); /* from @keymap_class */
+	data = JS_GetInstancePrivate(ctx, obj,
+				     (JSClass *) &keymap_class, NULL);
 
 	mem_free(data);
 }
