@@ -205,7 +205,7 @@ update_screen_driver(struct screen_driver *driver, struct option *term_spec)
 	}
 
 	if (utf8_io) {
-		driver->charsets[0] = get_opt_int_tree(term_spec, "charset");
+		driver->charsets[0] = get_opt_codepage_tree(term_spec, "charset");
 		if (driver->type == TERM_LINUX) {
 			if (get_opt_bool_tree(term_spec, "restrict_852"))
 				driver->frame = frame_restrict;
@@ -252,10 +252,9 @@ screen_driver_change_hook(struct session *ses, struct option *term_spec,
 	enum term_mode_type type = get_opt_int_tree(term_spec, "type");
 	struct screen_driver *driver;
 	unsigned char *name = term_spec->name;
-	int len = strlen(name);
 
 	foreach (driver, active_screen_drivers)
-		if (driver->type == type && !memcmp(driver->name, name, len)) {
+		if (driver->type == type && !strcmp(driver->name, name)) {
 			update_screen_driver(driver, term_spec);
 			break;
 		}
@@ -294,7 +293,7 @@ get_screen_driver(struct terminal *term)
 
 	foreach (driver, active_screen_drivers) {
 		if (driver->type != type) continue;
-		if (memcmp(driver->name, name, len + 1)) continue;
+		if (strcmp(driver->name, name)) continue;
 
 		/* Some simple probably useless MRU ;) */
 		move_to_top_of_list(active_screen_drivers, driver);
