@@ -1,5 +1,4 @@
 /* Terminal interface - low-level displaying implementation. */
-/* $Id: terminal.c,v 1.78.6.3 2005/05/01 21:15:33 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -15,12 +14,14 @@
 #include "elinks.h"
 
 #include "bookmarks/bookmarks.h"
-#include "main.h"
+#include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/select.h"
-#include "lowlevel/signals.h"
+#include "main/main.h"
+#include "main/object.h"
+#include "main/select.h"
 #include "osdep/osdep.h"
-#include "sched/session.h"
+#include "osdep/signals.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/event.h"
 #include "terminal/hardio.h"
@@ -30,7 +31,6 @@
 #include "terminal/window.h"
 #include "util/error.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
 #include "viewer/text/textarea.h"
 
@@ -42,10 +42,20 @@ static void check_if_no_terminal(void);
 
 
 void
-redraw_terminal_ev(struct terminal *term, int e)
+redraw_terminal(struct terminal *term)
 {
-	struct term_event ev = INIT_TERM_EVENT(e, term->width, term->height, 0);
+	struct term_event ev;
 
+	set_redraw_term_event(&ev, term->width, term->height);
+	term_send_event(term, &ev);
+}
+
+void
+redraw_terminal_cls(struct terminal *term)
+{
+	struct term_event ev;
+
+	set_resize_term_event(&ev, term->width, term->height);
 	term_send_event(term, &ev);
 }
 
@@ -85,8 +95,8 @@ init_term(int fdin, int fdout)
 
 	add_to_list(terminals, term);
 
-	set_handlers(fdin, (void (*)(void *)) in_term, NULL,
-		     (void (*)(void *)) destroy_terminal, term);
+	set_handlers(fdin, (select_handler_T) in_term, NULL,
+		     (select_handler_T) destroy_terminal, term);
 	return term;
 }
 
@@ -105,6 +115,7 @@ destroy_terminal(struct terminal *term)
 #ifdef CONFIG_BOOKMARKS
 	bookmark_auto_save_tabs(term);
 #endif
+	detach_downloads_from_terminal(term);
 
 	while (!list_empty(term->windows))
 		delete_window(term->windows.next);
@@ -151,7 +162,8 @@ destroy_all_terminals(void)
 static void
 check_if_no_terminal(void)
 {
-	terminate = list_empty(terminals);
+	program.terminate = list_empty(terminals)
+			    && !get_opt_bool("ui.sessions.keep_session_active");
 }
 
 void
@@ -178,14 +190,32 @@ unblock_terminal(struct terminal *term)
 {
 	close_handle((void *) (long) term->blocked);
 	term->blocked = -1;
-	set_handlers(term->fdin, (void (*)(void *)) in_term, NULL,
-		     (void (*)(void *)) destroy_terminal, term);
+	set_handlers(term->fdin, (select_handler_T) in_term, NULL,
+		     (select_handler_T) destroy_terminal, term);
 	unblock_itrm(term->fdin);
 	redraw_terminal_cls(term);
 	if (textarea_editor)	/* XXX */
 		textarea_edit(1, NULL, NULL, NULL, NULL);
 }
 
+#ifndef CONFIG_FASTMEM
+void
+assert_terminal_ptr_not_dangling(const struct terminal *suspect)
+{
+	struct terminal *term;
+
+	if (suspect == NULL)
+		return;
+
+	foreach (term, terminals) {
+		if (term == suspect)
+			return;
+	}
+
+	assertm(0, "Dangling pointer to struct terminal");
+}
+#endif /* !CONFIG_FASTMEM */
+
 void
 exec_on_terminal(struct terminal *term, unsigned char *path,
 		 unsigned char *delete, int fg)
@@ -238,12 +268,12 @@ exec_on_terminal(struct terminal *term, unsigned char *path,
 			if (fg == 1) {
 				term->blocked = blockh;
 				set_handlers(blockh,
-					     (void (*)(void *)) unblock_terminal,
+					     (select_handler_T) unblock_terminal,
 					     NULL,
-					     (void (*)(void *)) unblock_terminal,
+					     (select_handler_T) unblock_terminal,
 					     term);
 				set_handlers(term->fdin, NULL, NULL,
-					     (void (*)(void *)) destroy_terminal,
+					     (select_handler_T) destroy_terminal,
 					     term);
 				/* block_itrm(term->fdin); */
 			} else {
