@@ -1,5 +1,4 @@
 /* Very fast search_keyword_in_list. */
-/* $Id: fastfind.c,v 1.47 2003/10/20 08:42:30 zas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -69,7 +68,7 @@
 
 /* Define it to generate performance and memory usage statistics to stderr. */
 #if 0
-#define FASTFIND_DEBUG
+#define DEBUG_FASTFIND
 #endif
 
 /* Define whether to use 32 or 64 bits per compressed element. */
@@ -85,8 +84,8 @@
 /* Use only 32 bits per element, but has very low limits. */
 /* Adequate for ELinks tags search. */
 
-#define POINTER_INDEX_BITS	9	/* 512 */
-#define LEAFSET_INDEX_BITS	14	/* 16384 */
+#define POINTER_INDEX_BITS	10	/* 1024 */
+#define LEAFSET_INDEX_BITS	13	/* 8192 */
 #define COMP_CHAR_INDEX_BITS	7	/* 128	*/
 
 #define ff_node ff_node_c /* Both are 32 bits long. */
@@ -134,7 +133,7 @@ struct ff_node {
 #define FF_MAX_KEYS	(1  << POINTER_INDEX_BITS)
 #define FF_MAX_LEAFSETS ((1 << LEAFSET_INDEX_BITS) - 1)
 #define FF_MAX_CHARS	(1  << COMP_CHAR_INDEX_BITS)
-
+#define FF_MAX_KEYLEN	255
 
 struct ff_node_c {
 	unsigned int e:END_LEAF_BITS;
@@ -146,113 +145,191 @@ struct ff_node_c {
 	unsigned int ch:COMP_CHAR_INDEX_BITS;
 };
 
+struct ff_data {
+	void *pointer;
+	int keylen;
+};
 
 struct fastfind_info {
-	void **pointers;
-	int *keylen_list;
+	struct ff_data *data;
 
 	struct ff_node **leafsets;
 	struct ff_node *root_leafset;
 
-	int uniq_chars_count;
 	int min_key_len;
 	int max_key_len;
-	int count;
-	int case_sensitive;
-	int idxtab[FF_MAX_CHARS];
 
+	int uniq_chars_count;
+	int count;
 	int pointers_count;
 	int leafsets_count;
 
-#ifdef FASTFIND_DEBUG
-	unsigned long searches;
-	unsigned long found;
-	unsigned long itertmp;
-	unsigned long iterdelta;
-	unsigned long iterations;
-	unsigned long tests;
-	unsigned long teststmp;
-	unsigned long testsdelta;
-	unsigned long memory_usage;
-	unsigned long total_key_len;
-#endif
+	unsigned int case_aware:1;
+	unsigned int locale_indep:1;
+	unsigned int compress:1;
 
+	int idxtab[FF_MAX_CHARS];
 	unsigned char uniq_chars[FF_MAX_CHARS];
+
+#ifdef DEBUG_FASTFIND
+	struct {
+		unsigned long searches;
+		unsigned long found;
+		unsigned long itertmp;
+		unsigned long iterdelta;
+		unsigned long itermax;
+		unsigned long iterations;
+		unsigned long tests;
+		unsigned long teststmp;
+		unsigned long testsdelta;
+		unsigned long testsmax;
+		unsigned long memory_usage;
+		unsigned long total_key_len;
+		unsigned int  compressed_nodes;
+		unsigned char *comment;
+	} debug;
+#endif
 };
 
 
-#ifdef FASTFIND_DEBUG
+#ifdef DEBUG_FASTFIND
 /* These are for performance testing. */
-#define meminc(x, size) (x)->memory_usage += (size)
-#define testinc(x) (x)->tests++
-#define iterinc(x) (x)->iterations++
-#define foundinc(x) \
+#define FF_DBG_mem(x, size) (x)->debug.memory_usage += (size)
+#define FF_DBG_test(x) (x)->debug.tests++
+#define FF_DBG_iter(x) (x)->debug.iterations++
+#define FF_DBG_cnode(x) (x)->debug.compressed_nodes++
+#define FF_DBG_found(x) \
 	do { \
-		(x)->iterdelta += (x)->iterations - (x)->itertmp;	\
-		(x)->testsdelta += (x)->tests - (x)->teststmp;		\
-		(x)->found++;						\
+		unsigned long iter = (x)->debug.iterations - (x)->debug.itertmp;	\
+		unsigned long tests = (x)->debug.tests - (x)->debug.teststmp;		\
+											\
+		(x)->debug.iterdelta += iter;						\
+		(x)->debug.testsdelta += tests;						\
+		if (iter > (x)->debug.itermax)						\
+			(x)->debug.itermax = iter;					\
+		if (tests > (x)->debug.testsmax)					\
+			(x)->debug.testsmax = tests;					\
+		(x)->debug.found++;							\
 	} while (0)
-/* ACCounted IF ;-) */
-#define accif(x) testinc(x); if
+#define FF_DBG_comment(x, str) do { (x)->debug.comment = empty_string_or_(str); } while (0)
 
-#else /* !FASTFIND_DEBUG */
+/* Update search stats. */
+static void
+FF_DBG_search_stats(struct fastfind_info *info, int key_len)
+{
+	info->debug.searches++;
+	info->debug.total_key_len += key_len;
+	info->debug.teststmp = info->debug.tests;
+	info->debug.itertmp = info->debug.iterations;
+}
 
-#define meminc(x, size)
-#define testinc(x)
-#define iterinc(x)
-#define foundinc(x)
-#define accif(x) if
+/* Dump all stats. */
+static void
+FF_DBG_dump_stats(struct fastfind_info *info)
+{
+	fprintf(stderr, "------ FastFind Statistics ------\n");
+	fprintf(stderr, "Comment     : %s\n", info->debug.comment);
+	fprintf(stderr, "Case-aware  : %s\n", info->case_aware ? "yes" : "no");
+	fprintf(stderr, "Locale-indep: %s\n", info->locale_indep ? "yes" : "no");
+	fprintf(stderr, "Compress    : %s\n", info->compress ? "yes" : "no");
+	fprintf(stderr, "Uniq_chars  : %s\n", info->uniq_chars);
+	fprintf(stderr, "Uniq_chars #: %d/%d max.\n", info->uniq_chars_count, FF_MAX_CHARS);
+	fprintf(stderr, "Min_key_len : %d\n", info->min_key_len);
+	fprintf(stderr, "Max_key_len : %d\n", info->max_key_len);
+	fprintf(stderr, "Entries     : %d/%d max.\n", info->pointers_count, FF_MAX_KEYS);
+	fprintf(stderr, "Leafsets    : %d/%d max.\n", info->leafsets_count, FF_MAX_LEAFSETS);
+	if (info->compress)
+		fprintf(stderr, "C. leafsets : %u/%d (%0.2f%%)\n",
+			info->debug.compressed_nodes,
+			info->leafsets_count,
+			100 * (double) info->debug.compressed_nodes / info->leafsets_count);
+	fprintf(stderr, "Memory usage: %lu bytes (cost per entry = %0.2f bytes)\n",
+		info->debug.memory_usage, (double) info->debug.memory_usage / info->pointers_count);
+	fprintf(stderr, "Struct info : %zu bytes\n", sizeof(*info) - sizeof(info->debug));
+	fprintf(stderr, "Struct node : %zu bytes\n", sizeof(struct ff_node));
+	fprintf(stderr, "Struct cnode: %zu bytes\n", sizeof(struct ff_node_c));
+	fprintf(stderr, "Searches    : %lu\n", info->debug.searches);
+	fprintf(stderr, "Found       : %lu (%0.2f%%)\n",
+		info->debug.found, 100 * (double) info->debug.found / info->debug.searches);
+	fprintf(stderr, "Iterations  : %lu (%0.2f per search, %0.2f before found, %lu max)\n",
+		info->debug.iterations, (double) info->debug.iterations / info->debug.searches,
+		(double) info->debug.iterdelta / info->debug.found,
+		info->debug.itermax);
+	fprintf(stderr, "Tests       : %lu (%0.2f per search, %0.2f per iter., %0.2f before found, %lu max)\n",
+		info->debug.tests, (double) info->debug.tests / info->debug.searches,
+		(double) info->debug.tests / info->debug.iterations,
+		(double) info->debug.testsdelta / info->debug.found,
+		info->debug.testsmax);
+	fprintf(stderr, "Total keylen: %lu bytes (%0.2f per search, %0.2f per iter.)\n",
+		info->debug.total_key_len, (double) info->debug.total_key_len / info->debug.searches,
+		(double) info->debug.total_key_len / info->debug.iterations);
+	fprintf(stderr, "\n");
+}
 
-#endif
+#else /* !DEBUG_FASTFIND */
+
+#define FF_DBG_mem(x, size)
+#define FF_DBG_test(x)
+#define FF_DBG_iter(x)
+#define FF_DBG_cnode(x)
+#define FF_DBG_found(x)
+#define FF_DBG_comment(x, comment)
+#define FF_DBG_search_stats(info, key_len)
+#define FF_DBG_dump_stats(info)
+
+#endif /* DEBUG_FASTFIND */
 
 
 static struct fastfind_info *
-init_fastfind(int case_sensitive)
+init_fastfind(struct fastfind_index *index, enum fastfind_flags flags)
 {
-	struct fastfind_info *info = mem_calloc(1, sizeof(struct fastfind_info));
+	struct fastfind_info *info = mem_calloc(1, sizeof(*info));
 
+	index->handle = info;
 	if (!info) return NULL;
 
-	meminc(info, sizeof(struct fastfind_info) - sizeof(unsigned long) * 10);
-	/* Non sense to use that code if key length > 255 so... */
-	info->min_key_len = 255;
-	info->case_sensitive = case_sensitive;
+	info->min_key_len = FF_MAX_KEYLEN;
+	info->case_aware = !!(flags & FF_CASE_AWARE);
+	info->locale_indep = !!(flags & FF_LOCALE_INDEP);
+	info->compress = !!(flags & FF_COMPRESS);
+
+	FF_DBG_mem(info, sizeof(*info) - sizeof(info->debug));
+	FF_DBG_comment(info, index->comment);
 
 	return info;
 }
 
 /* Return 1 on success, 0 on allocation failure */
 static int
-add_to_pointers(void *p, int key_len, struct fastfind_info *info)
+alloc_ff_data(struct fastfind_info *info)
 {
-	void **pointers;
-	int *keylen_list;
-	int new_count = info->pointers_count + 1;
+	struct ff_data *data;
 
-	assert(new_count < FF_MAX_KEYS);
+	assert(info->count < FF_MAX_KEYS);
 	if_assert_failed return 0;
 
 	/* On error, cleanup is done by fastfind_done(). */
 
-	pointers = mem_realloc(info->pointers, new_count * sizeof(void *));
-	if (!pointers) return 0;
-	info->pointers = pointers;
-
-	keylen_list = mem_realloc(info->keylen_list, new_count * sizeof(int));
-	if (!keylen_list) return 0;
-	info->keylen_list = keylen_list;
-
-	meminc(info, sizeof(int) + sizeof(void *));
-
-	/* Record new pointer and key len, used in search */
-	info->pointers[info->pointers_count] = p;
-	info->keylen_list[info->pointers_count] = key_len;
-
-	info->pointers_count = new_count;
+	data = mem_calloc(info->count, sizeof(*data));
+	if (!data) return 0;
+	info->data = data;
+	FF_DBG_mem(info, info->count * sizeof(*data));
 
 	return 1;
 }
 
+/* Add pointer and its key length to correspondant arrays, incrementing
+ * internal counter. */
+static void
+add_to_ff_data(void *p, int key_len, struct fastfind_info *info)
+{
+	struct ff_data *data = &info->data[info->pointers_count++];
+
+	/* Record new pointer and key len, used in search */
+	data->pointer = p;
+	data->keylen = key_len;
+}
+
 /* Return 1 on success, 0 on allocation failure */
 static int
 alloc_leafset(struct fastfind_info *info)
@@ -266,17 +343,15 @@ alloc_leafset(struct fastfind_info *info)
 	/* info->leafsets[0] is never used since l=0 marks no leaf
 	 * in struct ff_node. That's the reason of that + 2. */
 	leafsets = mem_realloc(info->leafsets,
-			       sizeof(struct ff_node *)
-			       * (info->leafsets_count + 2));
+			       sizeof(*leafsets) * (info->leafsets_count + 2));
 	if (!leafsets) return 0;
 	info->leafsets = leafsets;
 
-	leafset = mem_calloc(info->uniq_chars_count,
-			     sizeof(struct ff_node));
+	leafset = mem_calloc(info->uniq_chars_count, sizeof(*leafset));
 	if (!leafset) return 0;
 
-	meminc(info, sizeof(struct ff_node *));
-	meminc(info, sizeof(struct ff_node) * info->uniq_chars_count);
+	FF_DBG_mem(info, sizeof(*leafsets));
+	FF_DBG_mem(info, sizeof(*leafset) * info->uniq_chars_count);
 
 	info->leafsets_count++;
 	info->leafsets[info->leafsets_count] = leafset;
@@ -284,49 +359,103 @@ alloc_leafset(struct fastfind_info *info)
 	return 1;
 }
 
-static int
+static inline int
 char2idx(unsigned char c, struct fastfind_info *info)
 {
-	register int idx;
+	unsigned char *idx = memchr(info->uniq_chars, c, info->uniq_chars_count);
 
-	for (idx = 0; idx < info->uniq_chars_count; idx++)
-		if (info->uniq_chars[idx] == c)
-			return idx;
+	if (idx) return (idx - info->uniq_chars);
 
 	return -1;
 }
 
-static void
+static inline void
 init_idxtab(struct fastfind_info *info)
 {
-	register int i;
+	int i;
 
 	for (i = 0; i < FF_MAX_CHARS; i++)
 		info->idxtab[i] = char2idx((unsigned char) i, info);
 }
 
-#define ifcase(c) (info->case_sensitive ? c : upcase(c))
+static inline void
+compress_node(struct ff_node *leafset, struct fastfind_info *info,
+	      int i, int pos)
+{
+	struct ff_node_c *new = mem_alloc(sizeof(*new));
 
-struct fastfind_info *
-fastfind_index(void (*reset)(void), struct fastfind_key_value *(*next)(void),
-	       int case_sensitive)
+	if (!new) return;
+
+	new->c = 1;
+	new->e = leafset[pos].e;
+	new->p = leafset[pos].p;
+	new->l = leafset[pos].l;
+	new->ch = pos;
+
+	mem_free_set(&info->leafsets[i], (struct ff_node *) new);
+	FF_DBG_cnode(info);
+	FF_DBG_mem(info, sizeof(*new));
+	FF_DBG_mem(info, sizeof(*leafset) * -info->uniq_chars_count);
+}
+
+static void
+compress_tree(struct ff_node *leafset, struct fastfind_info *info)
+{
+	int cnt = 0;
+	int pos = 0;
+	int i;
+
+	assert(info);
+	if_assert_failed return;
+
+	for (i = 0; i < info->uniq_chars_count; i++) {
+		if (leafset[i].c) continue;
+
+		if (leafset[i].l) {
+			/* There's a leaf leafset, descend to it and recurse */
+			compress_tree(info->leafsets[leafset[i].l], info);
+		}
+
+		if (leafset[i].l || leafset[i].e) {
+			cnt++;
+			pos = i;
+		}
+	}
+
+	if (cnt != 1 || leafset[pos].c) return;
+
+	/* Compress if possible ;) */
+	for (i = 1; i < info->leafsets_count; i++) {
+		if (info->leafsets[i] == leafset) {
+			compress_node(leafset, info, i, pos);
+			return;
+		}
+	}
+}
+
+#define ifcase(c) ( info->case_aware ? (c) : (info->locale_indep ? c_toupper(c) : toupper(c)) )
+
+struct fastfind_index *
+fastfind_index(struct fastfind_index *index, enum fastfind_flags flags)
 {
 	struct fastfind_key_value *p;
-	struct fastfind_info *info = init_fastfind(case_sensitive);
+	struct fastfind_info *info;
 
-	if (!info) goto alloc_error;
+	assert(index && index->reset && index->next);
+	if_assert_failed goto return_error;
 
-	assert(reset && next);
-	if_assert_failed goto alloc_error;
+	info = init_fastfind(index, flags);
+	if (!info) goto return_error;
 
 	/* First search min, max, count and uniq_chars. */
-	(*reset)();
-	while ((p = (*next)())) {
-		int key_len = strlen(p->key);
-		register int i;
+	index->reset();
 
-		assert(key_len); /* We do not want empty keys. */
-		if_assert_failed goto alloc_error;
+	while ((p = index->next())) {
+		int key_len = strlen(p->key);
+		int i;
+
+		assert(key_len > 0 && key_len <= FF_MAX_KEYLEN);
+		if_assert_failed goto return_error;
 
 		if (key_len < info->min_key_len)
 			info->min_key_len = key_len;
@@ -335,24 +464,19 @@ fastfind_index(void (*reset)(void), struct fastfind_key_value *(*next)(void),
 			info->max_key_len = key_len;
 
 		for (i = 0; i < key_len; i++) {
-			int j, k;
-
-			k = ifcase(p->key[i]);
-
-			assert(k < FF_MAX_CHARS);
-			if_assert_failed goto alloc_error;
-
 			/* ifcase() test should be moved outside loops but
 			 * remember we call this routine only once per list.
 			 * So I go for code readability vs performance here.
 			 * --Zas */
-			for (j = 0; j < info->uniq_chars_count; j++)
-				if (info->uniq_chars[j] == k)
-						break;
+			int k = ifcase(p->key[i]);
 
-			if (j >= info->uniq_chars_count) {
+			assert(k < FF_MAX_CHARS);
+			if_assert_failed goto return_error;
+
+			if (char2idx(k, info) == -1) {
 				assert(info->uniq_chars_count < FF_MAX_CHARS);
-				if_assert_failed goto alloc_error;
+				if_assert_failed goto return_error;
+
 				info->uniq_chars[info->uniq_chars_count++] = k;
 			}
 		}
@@ -360,21 +484,24 @@ fastfind_index(void (*reset)(void), struct fastfind_key_value *(*next)(void),
 		info->count++;
 	}
 
-	if (!info->count) return 0;
+	if (!info->count) return NULL;
 
 	init_idxtab(info);
 
 	/* Root leafset allocation */
-	if (!alloc_leafset(info)) goto alloc_error;
+	if (!alloc_leafset(info)) goto return_error;
 
 	info->root_leafset = info->leafsets[info->leafsets_count];
 
+	if (!alloc_ff_data(info)) goto return_error;
+
 	/* Build the tree */
-	(*reset)();
-	while ((p = (*next)())) {
+	index->reset();
+
+	while ((p = index->next())) {
 		int key_len = strlen(p->key);
 		struct ff_node *leafset = info->root_leafset;
-		register int i;
+		int i;
 
 #if 0
 		fprintf(stderr, "K: %s\n", p->key);
@@ -387,7 +514,7 @@ fastfind_index(void (*reset)(void), struct fastfind_key_value *(*next)(void),
 
 			if (leafset[idx].l == 0) {
 				/* There's no leaf yet */
-				if (!alloc_leafset(info)) goto alloc_error;
+				if (!alloc_leafset(info)) goto return_error;
 				leafset[idx].l = info->leafsets_count;
 			}
 
@@ -402,128 +529,85 @@ fastfind_index(void (*reset)(void), struct fastfind_key_value *(*next)(void),
 
 		/* Memorize pointer to data */
 		leafset[i].p = info->pointers_count;
-		if (!add_to_pointers(p->data, key_len, info))
-			goto alloc_error;
+		add_to_ff_data(p->data, key_len, info);
 	}
 
-	return info;
+	if (info->compress)
+		compress_tree(info->root_leafset, info);
 
-alloc_error:
-	fastfind_done(info);
+	return index;
+
+return_error:
+	fastfind_done(index);
 	return NULL;
 }
 
-void
-fastfind_node_compress(struct ff_node *leafset, struct fastfind_info *info)
-{
-	int cnt = 0;
-	int pos = 0;
-	register int i = 0;
+#undef ifcase
 
-	assert(info);
-	if_assert_failed return;
-
-	for (; i < info->uniq_chars_count; i++) {
-		if (leafset[i].c) continue;
-
-		if (leafset[i].l) {
-			/* There's a leaf leafset, descend to it and recurse */
-			fastfind_node_compress(info->leafsets[leafset[i].l],
-						info);
-		}
-
-		if (leafset[i].l || leafset[i].e) {
-			cnt++;
-			pos = i;
-		}
-	}
-
-	if (cnt != 1 || leafset[pos].c) return;
-
-	/* Compress if possible ;) */
-	for (i = 1; i < info->leafsets_count; i++)
-		if (info->leafsets[i] == leafset)
-			break;
-
-	if (i < info->leafsets_count) {
-		struct ff_node_c *new = mem_alloc(sizeof(struct ff_node_c));
-
-		if (!new) return;
-
-		new->c = 1;
-		new->e = leafset[pos].e;
-		new->p = leafset[pos].p;
-		new->l = leafset[pos].l;
-		new->ch = pos;
-
-		mem_free(info->leafsets[i]);
-		info->leafsets[i] = (struct ff_node *) new;
-		meminc(info, sizeof(struct ff_node_c));
-		meminc(info, sizeof(struct ff_node) * -info->uniq_chars_count);
-	}
-}
-
-void
-fastfind_index_compress(struct fastfind_info *info)
-{
-	assert(info);
-	if_assert_failed return;
-	fastfind_node_compress(info->root_leafset, info);
-}
 
 /* This macro searchs for the key in indexed list */
 #define FF_SEARCH(what) do {							\
-	register int i = 0;							\
+	int i;									\
 										\
-	for (; i < key_len; i++) {						\
+	for (i = 0; i < key_len; i++) {						\
 		int lidx, k = what;						\
 										\
-		iterinc(info);							\
+		FF_DBG_iter(info);						\
 										\
-		accif(info) (k >= FF_MAX_CHARS) return NULL;			\
+		FF_DBG_test(info);						\
+		if (k >= FF_MAX_CHARS) return NULL;				\
 		lidx = info->idxtab[k];						\
 										\
-		accif(info) (lidx < 0) return NULL;				\
+		FF_DBG_test(info);						\
+		if (lidx < 0) return NULL;					\
 										\
-		accif(info) (current->c) {					\
+		FF_DBG_test(info);						\
+		if (current->c) {						\
 			/* It is a compressed leaf. */				\
-			accif(info) (((struct ff_node_c *) current)->ch != lidx)\
+			FF_DBG_test(info);					\
+			if (((struct ff_node_c *) current)->ch != lidx)		\
 				return NULL;					\
 		} else {							\
 			current = &current[lidx];				\
 		}								\
 										\
-		accif(info) (current->e						\
-			     && key_len == info->keylen_list[current->p]) {	\
-			testinc(info);						\
-			foundinc(info);						\
-			return info->pointers[current->p];			\
+		FF_DBG_test(info);						\
+		if (current->e) {						\
+			struct ff_data *data = &info->data[current->p];		\
+										\
+			FF_DBG_test(info);					\
+			if (key_len == data->keylen) {				\
+				FF_DBG_found(info);				\
+				return data->pointer;				\
+			}							\
 		}								\
 										\
-		accif(info) (!current->l)					\
-				return NULL;					\
+		FF_DBG_test(info);						\
+		if (!current->l) return NULL;					\
 		current = (struct ff_node *) info->leafsets[current->l];	\
 	}									\
 } while (0)
 
 void *
-fastfind_search(unsigned char *key, int key_len, struct fastfind_info *info)
+fastfind_search(struct fastfind_index *index,
+		const unsigned char *key, int key_len)
 {
 	struct ff_node *current;
+	struct fastfind_info *info;
 
-	assert(info);
+	assert(index);
 	if_assert_failed return NULL;
 
-#ifdef FASTFIND_DEBUG
-	info->searches++;
-	info->total_key_len += key_len;
-	info->teststmp = info->tests;
-	info->itertmp = info->iterations;
-#endif
+	info = index->handle;
 
-   	accif(info) (!key) return NULL;
-	accif(info) (key_len > info->max_key_len) return NULL;
-	accif(info) (key_len < info->min_key_len) return NULL;
+	assertm(info, "FastFind index %s not initialized", index->comment);
+	if_assert_failed return NULL;
+
+	FF_DBG_search_stats(info, key_len);
+
+	FF_DBG_test(info); if (!key) return NULL;
+	FF_DBG_test(info); if (key_len > info->max_key_len) return NULL;
+	FF_DBG_test(info); if (key_len < info->min_key_len) return NULL;
 
 	current = info->root_leafset;
 
@@ -533,10 +617,14 @@ fastfind_search(unsigned char *key, int key_len, struct fastfind_info *info)
 	 * If you find a better way (same or better performance) then
 	 * propose it and be prepared to defend it. --Zas */
 
-	accif(info) (info->case_sensitive)
+	FF_DBG_test(info);
+	if (info->case_aware)
 		FF_SEARCH(key[i]);
 	else
-		FF_SEARCH(upcase(key[i]));
+		if (info->locale_indep)
+			FF_SEARCH(c_toupper(key[i]));
+		else
+			FF_SEARCH(toupper(key[i]));
 
 	return NULL;
 }
@@ -544,49 +632,28 @@ fastfind_search(unsigned char *key, int key_len, struct fastfind_info *info)
 #undef FF_SEARCH
 
 void
-fastfind_done(struct fastfind_info *info)
+fastfind_done(struct fastfind_index *index)
 {
+	struct fastfind_info *info;
+
+	assert(index);
+	if_assert_failed return;
+
+	info = index->handle;
 	if (!info) return;
 
-#ifdef FASTFIND_DEBUG
-	fprintf(stderr, "------ FastFind Statistics ------\n");
-	fprintf(stderr, "Uniq_chars  : %s\n", info->uniq_chars);
-	fprintf(stderr, "Uniq_chars #: %d/%d max.\n", info->uniq_chars_count, FF_MAX_CHARS);
-	fprintf(stderr, "Min_key_len : %d\n", info->min_key_len);
-	fprintf(stderr, "Max_key_len : %d\n", info->max_key_len);
-	fprintf(stderr, "Entries     : %d/%d max.\n", info->pointers_count, FF_MAX_KEYS);
-	fprintf(stderr, "FFleafsets  : %d/%d max.\n", info->leafsets_count, FF_MAX_LEAFSETS);
-	fprintf(stderr, "Memory usage: %lu bytes (cost per entry = %0.2f bytes)\n",
-		info->memory_usage, (double) info->memory_usage / info->pointers_count);
-	fprintf(stderr, "Struct node : %d bytes (normal) , %d bytes (compressed)\n",
-		sizeof(struct ff_node), sizeof(struct ff_node_c));
-	fprintf(stderr, "Searches    : %lu\n", info->searches);
-	fprintf(stderr, "Found       : %lu (%0.2f%%)\n",
-		info->found, 100 * (double) info->found / info->searches);
-	fprintf(stderr, "Iterations  : %lu (%0.2f per search, %0.2f before found)\n",
-		info->iterations, (double) info->iterations / info->searches,
-		(double) info->iterdelta / info->found);
-	fprintf(stderr, "Tests       : %lu (%0.2f per search, %0.2f per iter., %0.2f before found)\n",
-		info->tests, (double) info->tests / info->searches,
-		(double) info->tests / info->iterations,
-		(double) info->testsdelta / info->found);
-	fprintf(stderr, "Total keylen: %lu bytes (%0.2f per search, %0.2f per iter.)\n",
-		info->total_key_len, (double) info->total_key_len / info->searches,
-		(double) info->total_key_len / info->iterations);
-#endif
+	FF_DBG_dump_stats(info);
 
-	if (info->pointers) mem_free(info->pointers);
-	if (info->keylen_list) mem_free(info->keylen_list);
+	mem_free_if(info->data);
 	while (info->leafsets_count) {
-		if (info->leafsets[info->leafsets_count])
-			mem_free(info->leafsets[info->leafsets_count]);
+		mem_free_if(info->leafsets[info->leafsets_count]);
 		info->leafsets_count--;
 	}
-	if (info->leafsets) mem_free(info->leafsets);
+	mem_free_if(info->leafsets);
 	mem_free(info);
-}
 
-#undef ifcase
+	index->handle = NULL;
+}
 
 
 /* EXAMPLE */
@@ -695,13 +762,14 @@ next_in_list(void)
 	return &kv;
 }
 
+static struct fastfind_index ff_index
+	= INIT_FASTFIND_INDEX("example", reset_list, next_in_list);
 
 int
 main(int argc, char **argv)
 {
 	unsigned char *key = argv[1];
 	struct list *result;
-	struct fastfind_info *info;
 
 	if (!key || !*key) {
 		fprintf(stderr, "Usage: fastfind keyword\n");
@@ -710,22 +778,19 @@ main(int argc, char **argv)
 
 	fprintf(stderr, "---------- INDEX PHASE -----------\n");
 	/* Mandatory */
-	info = fastfind_index(&reset_list, &next_in_list);
-
-	fprintf(stderr, "--------- COMPRESS PHASE ---------\n");
-	/* Highly recommended but optional. */
-	fastfind_index_compress(info);
+	fastfind_index(&ff_index, FF_COMPRESS);
 
 	fprintf(stderr, "---------- SEARCH PHASE ----------\n");
 	/* Without this one ... */
-	result = (struct list *) fastfind_search(key, strlen(key), info);
+	result = (struct list *) fastfind_search(&ff_index, key, strlen(key));
 
 	if (result)
 		fprintf(stderr, " Found: '%s' -> %d\n", result->tag, result->val);
 	else
 		fprintf(stderr, " Not found: '%s'\n", key);
 
-	fastfind_done(info);
+	fprintf(stderr, "---------- CLEANUP PHASE ----------\n");
+	fastfind_done(&ff_index);
 
 	exit(0);
 }
