@@ -921,6 +921,14 @@ activate_link(struct session *ses, struct document_view *doc_view,
 			return FRAME_EVENT_REFRESH;
 		}
 
+		/* @link_fc->type must be FC_RADIO, then.  First turn
+		 * this one on, and then turn off all the other radio
+		 * buttons in the group.  Do it in this order because
+		 * further @find_form_state calls may reallocate
+		 * @doc_view->vs->form_info[] and thereby make the @fs
+		 * pointer invalid.  This also allows us to re-use
+		 * @fs in the loop. */
+		fs->state = 1;
 		foreach (form, doc_view->document->forms) {
 			struct form_control *fc;
 
@@ -929,15 +937,13 @@ activate_link(struct session *ses, struct document_view *doc_view,
 
 			foreach (fc, form->items) {
 				if (fc->type == FC_RADIO
-				    && !xstrcmp(fc->name, link_fc->name)) {
-					struct form_state *frm_st;
-
-					frm_st = find_form_state(doc_view, fc);
-					if (frm_st) frm_st->state = 0;
+				    && !xstrcmp(fc->name, link_fc->name)
+				    && fc != link_fc) {
+					fs = find_form_state(doc_view, fc);
+					if (fs) fs->state = 0;
 				}
 			}
 		}
-		fs->state = 1;
 
 		break;
 
@@ -1093,7 +1099,6 @@ try_document_key(struct session *ses, struct document_view *doc_view,
 		 struct term_event *ev)
 {
 	long key;
-	int passed = -1;
 	int i; /* GOD I HATE C! --FF */ /* YEAH, BRAINFUCK RULEZ! --pasky */
 
 	assert(ses && doc_view && doc_view->document && doc_view->vs && ev);
@@ -1108,24 +1113,23 @@ try_document_key(struct session *ses, struct document_view *doc_view,
 	 * key we test.. */
 	key = get_kbd_key(ev);
 
-	for (i = 0; i < doc_view->document->nlinks; i++) {
+	i = doc_view->vs->current_link + 1;
+	for (; i < doc_view->document->nlinks; i++) {
 		struct link *link = &doc_view->document->links[i];
 
 		if (key == link->accesskey) {	/* FIXME: key vs unicode ... */
-			if (passed != i && i <= doc_view->vs->current_link) {
-				/* This is here in order to rotate between
-				 * links with same accesskey. */
-				if (passed < 0)	passed = i;
-				continue;
-			}
 			ses->kbdprefix.repeat_count = 0;
 			goto_link_number_do(ses, doc_view, i);
 			return FRAME_EVENT_REFRESH;
 		}
+	}
+	for (i = 0; i <= doc_view->vs->current_link; i++) {
+		struct link *link = &doc_view->document->links[i];
 
-		if (i == doc_view->document->nlinks - 1 && passed >= 0) {
-			/* Return to the start. */
-			i = passed - 1;
+		if (key == link->accesskey) {	/* FIXME: key vs unicode ... */
+			ses->kbdprefix.repeat_count = 0;
+			goto_link_number_do(ses, doc_view, i);
+			return FRAME_EVENT_REFRESH;
 		}
 	}
 
